# Endless City v2 - Data-driven format

- This file lays out the format for the data storages used to define the generated road tiles and structures. These details are also included in the `load` function for the city packs.
  - Only the fields which should be user-edited are shown - nothing else should be added, though temp data can be created for using duplicate data, and then destroyed again.
  - User-defined structures *__must__* be added to their correct storages via a function run from the `#end_cit:structure_config` tag (user defined functions can be freely added to this, beyond the built-in optional module functions) to ensure they are loaded at the correct stage in pre-generation. Do *__not__* modify the storages from other load functions et cetera as this can and will cause issues.

## City structures

### Storage `end_cit:structures`
```apache
{
  <str: Small|Medium|Long|Large|SideDecor|ArchDecor>: 
  {
    Structures: 
    [
      <compound - entry>
    ]
  }
}
```
Entry format:
```apache
{
  Name <str>  # Namespaced id for structure (.nbt) file

  # Offset from centre (where the marker entity which spawns the structure is placed) and the bottom north-west corner
  x_offset <int>
  y_offset <int>
  z_offset <int>

  # Structure file dimensions (as shown in structure block)
  x_size <int> 
  y_size <int>
  z_size <int>

  failChance <int> 
    # The chance (out of 100) for the structure to not place
  decayChance <int> 
    # The chance (out of 100) for the structure to decay down to the next size level

  randomRotation <bool>
    # Whether or not the structure should rotate randomly (about the marker entity) or inherit its rotation from the marker entity
  randomPosition <bool>
    # Whether the structure should randomly offset by up to 2 blocks in the x/y directions

  weight <int>
    # Randomisation weight for this tile to generate from the list of tiles in this size

  # Stacked building configuration (multiple randomised layers)
  Stack <compound> (optional):
  {
    Limit <str>
      # namespaced id for the predicate which limits the structure height
    Offset <int>
      # the vertical offset to place each subsequent layer
    failChance <int>
      # the chance for each layer to be the final layer (i.e. the stack terminates)
    minStackCount <int>
      # the minimum number of layers to place

    # List of namespaced ids for structure (.nbt) files
    Structures: 
    [
      <str - structure file id>
    ]
  }

  PostProcessors <list compound> (optional):
  [
    <compound - Post processor>
  ]
}
```
Post processor format:
```apache
{
  type <enum replace_random|replace_random_rotated>
    # The processor type
    # 

  # Relative position of the block to be replaced, from the bottom north-west corner of the structure 
  x <int>
  y <int>
  z <int>

  require_air <bool> (optional)
    # Whether or not the block replacement requires the block being replaced to be air (defaults to false)

  # List of namespaced block ids to select from to replace the given block with
  blocks: 
  [
    <str - block id>
  ]

  # IF (type: replace_random_rotated)
  # List of block-data entries - 4 entries, corresponding to the different 
  #   e.g. ["[rotation=0]","[rotation=4]","[rotation=8]","[rotation=12]"]
  data: 
  [
    <str>
  ]
}
```